@@ -51,9 +51,9 @@ from RMutils.util_RM import measure_qu_complexity
 from RMutils.util_RM import measure_fdf_complexity
 from RMutils.util_misc import nanmedian
 from RMutils.util_misc import toscalar
-from RMutils.util_misc import create_frac_spectra
-from RMutils.util_misc import poly5
-from RMutils.util_misc import MAD
+from RMutils.util_misc import create_frac_spectra,calculate_StokesI_model
+from RMutils.util_misc import poly5,powerlaw_poly5
+from RMutils.util_misc import MAD,renormalize_StokesI_model
 from RMutils.util_plotTk import plot_Ipqu_spectra_fig
 from RMutils.util_plotTk import plot_rmsf_fdf_fig
 from RMutils.util_plotTk import plot_complexity_fig
@@ -70,7 +70,8 @@ C = 2.997924538e8 # Speed of light [m/s]
 def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
                 nSamples=10.0, weightType="variance", fitRMSF=False,
                 noStokesI=False, phiNoise_radm2=1e6, nBits=32, showPlots=False,
-                debug=False, verbose=False, log=print,units='Jy/beam', prefixOut="prefixOut", saveFigures=None):
+                debug=False, verbose=False, log=print,units='Jy/beam', 
+                prefixOut="prefixOut", saveFigures=None,fit_function='log'):
     """Run RM synthesis on 1D data.
 
     Args:
@@ -152,7 +153,7 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
 
     # Fit the Stokes I spectrum and create the fractional spectra
     IModArr, qArr, uArr, dqArr, duArr, fitDict = \
-             create_frac_spectra(freqArr  = freqArr_GHz,
+             create_frac_spectra(freqArr  = freqArr_Hz,
                                  IArr     = IArr,
                                  QArr     = QArr,
                                  UArr     = UArr,
@@ -161,17 +162,19 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
                                  dUArr    = dUArr,
                                  polyOrd  = polyOrd,
                                  verbose  = True,
-                                 debug    = debug)
+                                 debug    = debug,
+                             fit_function = fit_function)
              
     dquArr = (dqArr + duArr)/2.0
+    dquArr= np.where(np.isfinite(dquArr),dquArr,np.nan)
 
 
     # Plot the data and the Stokes I model fit
     if verbose: log("Plotting the input data and spectral index fit.")
     freqHirArr_Hz =  np.linspace(freqArr_Hz[0], freqArr_Hz[-1], 10000)
-    IModHirArr = poly5(fitDict["p"])(freqHirArr_Hz/1e9)
+    IModHirArr = calculate_StokesI_model(fitDict,freqHirArr_Hz)
     if showPlots or saveFigures:
-        specFig = plt.figure(figsize=(12.0, 8))
+        specFig = plt.figure(facecolor='w',figsize=(12.0, 8))
         plot_Ipqu_spectra_fig(freqArr_Hz     = freqArr_Hz,
                               IArr           = IArr,
                               qArr           = qArr,
@@ -201,7 +204,7 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
 
     # DEBUG (plot the Q, U and average RMS spectrum)
     if debug:
-        rmsFig = plt.figure(figsize=(12.0, 8))
+        rmsFig = plt.figure(facecolor='w',figsize=(12.0, 8))
         ax = rmsFig.add_subplot(111)
         ax.plot(freqArr_Hz/1e9, dQUArr, marker='o', color='k', lw=0.5,
                 label='rms <QU>')
@@ -292,15 +295,19 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
     cputime = (endTime - startTime)
     if verbose: log("> RM-synthesis completed in %.2f seconds." % cputime)
 
+
     # Determine the Stokes I value at lam0Sq_m2 from the Stokes I model
     # Multiply the dirty FDF by Ifreq0 to recover the PI
     freq0_Hz = C / m.sqrt(lam0Sq_m2)
-    Ifreq0 = poly5(fitDict["p"])(freq0_Hz/1e9)
+    Ifreq0 = calculate_StokesI_model(fitDict,freq0_Hz)
     dirtyFDF *= (Ifreq0)    # FDF is in fracpol units initially, convert back to flux
 
+    #Need to renormalize the Stokes I parameters here to the actual reference frequency.
+    fitDict=renormalize_StokesI_model(fitDict,freq0_Hz)
+
     # Calculate the theoretical noise in the FDF !!Old formula only works for wariance weights!
     weightArr = np.where(np.isnan(weightArr), 0.0, weightArr)
-    dFDFth = Ifreq0*np.sqrt( np.sum(weightArr**2 * np.nan_to_num(dquArr)**2) / (np.sum(weightArr))**2 )
+    dFDFth = Ifreq0*np.sqrt( np.nansum(weightArr**2 * np.nan_to_num(dquArr)**2) / (np.sum(weightArr))**2 )
 
 
     # Measure the parameters of the dirty FDF
@@ -321,10 +328,11 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
     mDict["dQU"] = toscalar(nanmedian(dQUArr))
     mDict["dFDFth"] = toscalar(dFDFth)
     mDict["units"] = units
+    mDict['polyOrd'] = fitDict['polyOrd']
     
-    if fitDict["fitStatus"] >= 128:
+    if (fitDict["fitStatus"] >= 128) and verbose:
         log("WARNING: Stokes I model contains negative values!")
-    elif fitDict["fitStatus"] >= 64:
+    elif (fitDict["fitStatus"] >= 64) and verbose:
         log("Caution: Stokes I model has low signal-to-noise.")
 
 
@@ -407,6 +415,7 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
        log('sigma_add(u) = %.4g (+%.4g, -%.4g)' % (mDict["sigmaAddU"],
                                             mDict["dSigmaAddPlusU"],
                                             mDict["dSigmaAddMinusU"]))
+       log('Fitted polynomial order = {} '.format(mDict['polyOrd']))
        log()
        log('-'*80)
 
@@ -414,7 +423,7 @@ def run_rmsynth(data, polyOrd=3, phiMax_radm2=None, dPhi_radm2=None,
 
     # Plot the RM Spread Function and dirty FDF
     if showPlots or saveFigures:
-        fdfFig = plt.figure(figsize=(12.0, 8))
+        fdfFig = plt.figure(facecolor='w',figsize=(12.0, 8))
         plot_rmsf_fdf_fig(phiArr     = phiArr_radm2,
                           FDF        = dirtyFDF,
                           phi2Arr    = phi2Arr_radm2,
@@ -547,8 +556,8 @@ def main():
     # Help string to be shown using the -h option
     descStr = """
     Run RM-synthesis on Stokes I, Q and U spectra (1D) stored in an ASCII
-    file. The Stokes I spectrum is first fit with a polynomial and the
-    resulting model used to create fractional q = Q/I and u = U/I spectra.
+    file. The Stokes I spectrum is first fit with a polynomial or power law 
+    and the resulting model used to create fractional q = Q/I and u = U/I spectra.
 
     The ASCII file should the following columns, in a space separated format:
     [freq_Hz, I, Q, U, I_err, Q_err, U_err]
@@ -584,8 +593,10 @@ def main():
                         help="number of samples across the RMSF lobe [10].")
     parser.add_argument("-w", dest="weightType", default="variance",
                         help="weighting [inverse 'variance'] or 'uniform' (all 1s).")
+    parser.add_argument("-f", dest="fit_function", type=str, default="log",
+                        help="Stokes I fitting function: 'linear' or ['log'] polynomials.")
     parser.add_argument("-o", dest="polyOrd", type=int, default=2,
-                        help="polynomial order to fit to I spectrum [2].")
+                        help="polynomial order to fit to I spectrum: 0-5 supported, 2 is default.\nSet to negative number to enable dynamic order selection.")
     parser.add_argument("-i", dest="noStokesI", action="store_true",
                         help="ignore the Stokes I spectrum [False].")
     parser.add_argument("-b", dest="bit64", action="store_true",
@@ -631,7 +642,7 @@ def main():
                 units          = args.units,
                 prefixOut      = prefixOut,
                 saveFigures    = args.saveOutput,
-                )
+                fit_function   = args.fit_function)
 
     if args.saveOutput:
         saveOutput(mDict, aDict, prefixOut, verbose)
