@@ -380,11 +380,13 @@ def plot_pqu_vs_lamsq_ax(ax, lamSqArr_m2, qArr, uArr, pArr=None, dqArr=None,
     # Calculate p and errors
     if pArr is None:
         pArr = np.sqrt(qArr**2.0 + uArr**2.0 )
+        pArr= np.where(np.isfinite(pArr),pArr,np.nan)
     if dpArr is None:
         if dqArr is None or duArr is None:
             dpArr = None
         else:
             dpArr = np.sqrt(dqArr**2.0 + duArr**2.0 )
+            dpArr= np.where(np.isfinite(dpArr),dpArr,np.nan)
         
     # Plot p, q, u versus lambda^2
 #    """
@@ -1326,7 +1328,7 @@ def plot_Ipqu_spectra_fig(freqArr_Hz, IArr, qArr, uArr, dIArr=None,
 
     # Default to a pyplot figure
     if fig==None:
-        fig = plt.figure(figsize=(12.0, 8))
+        fig = plt.figure(facecolor='w',figsize=(12.0, 8))
 
     # Default to non-high-resolution inputs
     if freqHirArr_Hz is None:
@@ -1531,7 +1533,7 @@ def plot_rmsf_fdf_fig(phiArr, FDF, phi2Arr, RMSFArr, fwhmRMSF=None,
 
     # Default to a pyplot figure
     if fig==None:
-        fig = plt.figure(figsize=(12.0, 8))
+        fig = plt.figure(facecolor='w',figsize=(12.0, 8))
     # Plot the RMSF
     ax1 = fig.add_subplot(211)    
     plot_RMSF_ax(ax=ax1,
@@ -1668,7 +1670,7 @@ def plot_cleanFDF_fig(phiArr, cleanFDFArr, ccFDFArr=None,
     
     # Default to a pyplot figure
     if fig==None:
-        fig = plt.figure(figsize=(12.0, 8))
+        fig = plt.figure(facecolor='w',figsize=(12.0, 8))
 
     # Plot the clean FDF
     ax1 = fig.add_subplot(211)    
@@ -1929,7 +1931,7 @@ def plot_complexity_fig(xArr, qArr, dqArr, sigmaAddqArr, chiSqRedqArr,
 
     # Default to a pyplot figure
     if fig==None:
-        fig = plt.figure(figsize=(16.0, 8.0))
+        fig = plt.figure(facecolor='w',figsize=(16.0, 8.0))
     
     # Plot the data and the +/- 1-sigma levels
     ax1 = fig.add_subplot(231)
@@ -1956,10 +1958,10 @@ def plot_complexity_fig(xArr, qArr, dqArr, sigmaAddqArr, chiSqRedqArr,
     nBins = 15
     yMin = np.nanmin([np.nanmin(qArr), np.nanmin(uArr)])
     yMax = np.nanmax([np.nanmax(qArr), np.nanmax(uArr)])
-    n, b, p = ax2.hist(qArr, nBins, range=(yMin, yMax), normed=1,
+    n, b, p = ax2.hist(qArr, nBins, range=(yMin, yMax), density=1,
                        histtype='step', color='tab:blue', linewidth=1.0)
     ax2.plot(xNorm, yNorm, color='k', linestyle="--", linewidth=1.5)
-    n, b, p = ax2.hist(uArr, nBins, range=(yMin, yMax), normed=1,
+    n, b, p = ax2.hist(uArr, nBins, range=(yMin, yMax), density=1,
                        histtype='step', color='tab:red', linewidth=1.0)
     ax2.axvline(med, color='grey', zorder=11)
     ax2.set_title(r'Distribution of Data vs Normal')
